@@ -20,6 +20,8 @@ def parse(xml):
     :returns: DOM of the parsed XML
 
     """
+    if isinstance(xml, unicode):
+        xml = xml.encode("utf-8")
     return ET.fromstring(xml)
 
 
@@ -306,6 +308,12 @@ class Field(Config):
         is already set by the underlying FormAlchemy library by checking
         if the database field is 'NOT NULL'. Defaults to False"""
 
+        self.desired= entity.attrib.get('desired', 'false') == 'true'
+        """Flag to mark the field as a desired field. If this tag is
+        set an additional rule will be added to the field and an star
+        symbol is rendered at the label of the field. Defaults to
+        False"""
+
         self.readonly = entity.attrib.get('readonly', 'false') == 'true'
         """Flag to set the field as a readonly field. If set the field
         will be rendered as a simple textfield which does not allow to
@@ -339,12 +347,30 @@ class Field(Config):
         # Get rules
         self.rules = []
         parser = Parser()
+        # Add automatic genertated rules based on the required or
+        # desired flag
+        if self.required:
+            expr = "bool($%s)" % self.name
+            msg = "This field is required. You need to provide a value"
+            mode = "pre"
+            expr = parser.parse(expr)
+            self.rules.append(Rule(expr, msg, mode))
+        if self.desired:
+            expr = "bool($%s)" % self.name
+            msg = "This field is desired. Pleas provide a value"
+            mode = "pre"
+            triggers = "warning"
+            expr = parser.parse(expr)
+            self.rules.append(Rule(expr, msg, mode, triggers))
+        # Add rules added the the field.
         for rule in self._tree.findall('rule'):
             expr = rule.attrib.get('expr')
             msg = rule.attrib.get('msg')
             mode = rule.attrib.get('mode')
+            triggers = rule.attrib.get('triggers')
             expr = parser.parse(expr)
-            self.rules.append(Rule(expr, msg, mode))
+            self.rules.append(Rule(expr, msg, mode, triggers))
+
 
 
 class Renderer(Config):
