@@ -3,7 +3,6 @@ import datetime
 import sqlalchemy as sa
 from formencode import htmlfill, variabledecode
 from formbar.renderer import FormRenderer, get_renderer
-from formbar.rules import Rule, Parser
 
 log = logging.getLogger(__name__)
 
@@ -125,10 +124,16 @@ class Form(object):
         """Dictionary with external provided custom renderers."""
         self.fields = self._build_fields()
         """Dictionary with fields."""
-        self.data = self.serialize(self._get_data_from_item())
-        """After submission this Dictionary will contain either the
-        validated data on successfull validation or the origin submitted
-        data."""
+        self.data = {}
+        """After submission this Dictionary will contain the
+        validated data on successfull validation. Else this Dictionary
+        will be empty"""
+        self.submitted_data = {}
+        """The submitted data from the user. If validation fails, then
+        this values are used to rerender the form."""
+        self.loaded_data = self.serialize(self._get_data_from_item())
+        """This is the initial data loaded from the given item. Used to
+        render the readonly forms"""
 
     def _get_data_from_item(self):
         values = {}
@@ -141,6 +146,39 @@ class Form(object):
                 values[name] = None
         return values
 
+    def _filter_values(self, values):
+        """Will return a filtered dictionary of the given values
+        dictionary which only contains values which are actually part of
+        the current form.
+
+        :values: Dicionary with unfilterd values
+        :returns: Dicionary with filtered values
+
+        """
+        filtered = {}
+        for fieldname, field in self.fields.iteritems():
+            if values.has_key(fieldname):
+                filtered[fieldname] = values[fieldname]
+        return filtered
+
+    def deserialize(self, data):
+        """Returns a dictionary with pythonized data data. Usually this
+        is the submitted data coming from a form. The dictionary will
+        include all values provided in the initial data dictionary
+        converted into python datatype.
+
+        :data: Dictionary with serialized data
+        :returns: Dictionary with deserialized data
+
+        """
+        deserialized = {}
+        for fieldname, value in self._filter_values(data).iteritems():
+            field = self.fields.get(fieldname)
+            if field and not field.is_readonly():
+                # Only add the value if the field is not marked as readonly
+                deserialized[fieldname] = self._to_python(field, data.get(field.name))
+        log.debug("Deserialized values: %s" % deserialized)
+        return deserialized
 
     def serialize(self, data):
         """Returns a dictionary with serialized data from the forms
@@ -155,38 +193,13 @@ class Form(object):
 
         """
         serialized = {}
-        if not data:
-            return serialized
-        for name, field in self.fields.iteritems():
-            ftype = field.get_type()
-            try:
-                value = data.get(name)
-                if value is None:
-                    serialized[name] = ""
-                elif isinstance(value, list):
-                    vl = []
-                    for v in value:
-                        try:
-                            vl.append(v.id)
-                        except AttributeError:
-                            vl.append(v)
-                    serialized[name] = vl
-                else:
-                    try:
-                        serialized[name] = value.id
-                    except AttributeError:
-                        if ftype == "time":
-                            td = datetime.timedelta(seconds=int(value))
-                            d = datetime.datetime(1, 1, 1) + td
-                            serialized[name] = "%02d:%02d:%02d" % (d.hour, d.minute, d.second)
-                        elif ftype == "datetime":
-                            serialized[name] = value.strftime("%Y-%m-%d %H:%M:%S")
-                        else:
-                            serialized[name] = value
-            except AttributeError:
-                log.warning('Can not get value for field "%s". '
-                            'The field is no attribute of the item' % name)
-                serialized[name] = ""
+        for fieldname, value in self._filter_values(data).iteritems():
+            field = self.fields.get(fieldname)
+            #if field and not field.is_readonly():
+            #    # Only add the value if the field is not marked as readonly
+            #    serialized[fieldname] = self._from_python(field, value)
+            serialized[fieldname] = self._from_python(field, value)
+        log.debug("Serialized values: %s" % serialized)
         return serialized
 
     def _build_fields(self):
@@ -207,6 +220,13 @@ class Form(object):
                 return True
         return False
 
+    def has_warnings(self):
+        """Returns True if one of the fields in the form has warnings"""
+        for field in self.fields.values():
+            if len(field.get_warnings()) > 0:
+                return True
+        return False
+
     def get_errors(self, page=None):
         """Returns a dictionary of all errors in the form. If page
         parameter is given, then only the errors for fields on the given
@@ -228,6 +248,27 @@ class Form(object):
                 errors[field.name] = field.get_errors()
         return errors
 
+    def get_warnings(self, page=None):
+        """Returns a dictionary of all warnings in the form. If page
+        parameter is given, then only the warnings for fields on the given
+        page are returned. This dictionary will contain the warnings if
+        the validation fails. The key of the dictionary is the fieldname
+        of the field.  As a field can have more than one warning the value
+        is a list.
+
+        :page: Name of the page
+        :returns: Dictionary with warnings
+        """
+        if page is not None:
+            fields_on_page = self._config.get_fields(page)
+
+        warnings = {}
+        for field in self.fields.values():
+            if page is not None and field.name not in fields_on_page: continue
+            if len(field.get_warnings()) > 0:
+                warnings[field.name] = field.get_warnings()
+        return warnings
+
     def get_field(self, name):
         return self.fields[name]
 
@@ -243,35 +284,20 @@ class Form(object):
 
         """
         self.current_page = page
-        renderer = FormRenderer(self, self._translate)
-        form = renderer.render(buttons)
 
-        # If we have a POST request than the user has sent modfied data.
-        # The content of self.values depends on the validation.
-        # If the form is validated and the form contains no erros
-        # than we use the serialized values of self.data which includes
-        # the converted values.
-        # In the other cases return the self.data attribute which the
-        # submitted error data on POST.
-        if self._request and self._request.POST:
-            if self.validated and not self.has_errors():
-                item_values = self.serialize(self.data)
-            else:
-                item_values = self.data
-        # If we have a GET request than the user has loaded a exising
-        # item (or wants to create a new one). In this case we will need
-        # to get the initial data of the item an merge it with the
-        # provided values. This data is already serialized in the
-        # self.data attribute.
+        if self.submitted_data:
+            item_values = self.submitted_data
         else:
-            item_values = self.data
-
+            item_values = self.loaded_data
         # Merge the items_values with the extra provided values. Extra
         # values will overwrite the item_values.
         values = dict(item_values.items() + values.items())
 
         # Add csrf_token to the values dictionary
         values['csrf_token'] = self._csrf_token
+
+        renderer = FormRenderer(self, self._translate)
+        form = renderer.render(values=values, buttons=buttons)
         return htmlfill.render(form, values)
 
     def _add_error(self, fieldname, error):
@@ -282,8 +308,17 @@ class Form(object):
         else:
             field.add_error(error)
 
-    def _convert(self, field, value):
-        """Returns a converted value depending of the fields datatype
+    def _add_warning(self, fieldname, warning):
+        field = self.get_field(fieldname)
+        if isinstance(warning, list):
+            for war in warning:
+                field.add_warning(war)
+        else:
+            field.add_warning(warning)
+
+    def _to_python(self, field, value):
+        """Returns a value of python datatype converted from the
+        stringvalue depending of the fields datatype
 
         :field: configuration of the field
         :value: value to be converted
@@ -436,6 +471,46 @@ class Form(object):
                   % (converted, type(converted), field.name, dtype))
         return converted
 
+    def _from_python(self, field, value):
+        """@todo: Docstring for _from_python.
+
+        :field: @todo
+        :value: @todo
+        :returns: @todo
+
+        """
+        serialized = ""
+        ftype = field.get_type()
+        try:
+            if value is None:
+                serialized = ""
+            elif isinstance(value, basestring):
+                serialized = value
+            elif isinstance(value, list):
+                vl = []
+                for v in value:
+                    try:
+                        vl.append(v.id)
+                    except AttributeError:
+                        vl.append(v)
+                serialized = vl
+            else:
+                try:
+                    serialized = value.id
+                except AttributeError:
+                    if ftype == "time":
+                        td = datetime.timedelta(seconds=int(value))
+                        d = datetime.datetime(1, 1, 1) + td
+                        serialized = "%02d:%02d:%02d" % (d.hour, d.minute, d.second)
+                    elif ftype == "datetime":
+                        serialized = value.strftime("%Y-%m-%d %H:%M:%S")
+                    else:
+                        serialized = value
+        except AttributeError:
+            log.warning('Can not get value for field "%s". '
+                        'The field is no attribute of the item' % field.name)
+        return serialized
+
     def validate(self, submitted):
         """Returns True if the validation succeeds else False.
         Validation of the data happens in three stages:
@@ -459,50 +534,35 @@ class Form(object):
 
         """
 
-        dirty_submitted = variabledecode.variable_decode(submitted)
-        # This dictionary will contain the converted data
-        values = {}
+        if not submitted:
+            unvalidated = self.loaded_data
+        else:
+            self.submitted_data = variabledecode.variable_decode(submitted)
+            unvalidated = self.submitted_data
+        converted = self.deserialize(unvalidated)
 
-        # 1. Collect all configured fields from form configuration and
-        # remove out values from the submitted data which are not part
-        # of the form.
-        fields_all = []
-        submitted = {}
-        log.debug('Submitted values: %s' % dirty_submitted)
-        for fieldname, field in self.fields.iteritems():
-            fields_all.append((fieldname, self.fields[fieldname]))
-            if dirty_submitted.has_key(fieldname):
-                submitted[fieldname] = dirty_submitted[fieldname]
-
-        # 2. Convert all values
-        for fieldname, field in fields_all:
-            # Basic type conversations, Defaults to String
-            if not field.is_readonly():
-                # Only add the value if the field is not marked as readonly
-                values[fieldname] = self._convert(field, submitted.get(fieldname))
-
-        # 3. Validate the fields. Ignore fields which are disabled in
-        # conditionals
-        # First get list of fields which are still in the form after
-        # conditionals has be evaluated
-        fields_to_check = self._config.get_fields(values=values,
+        # Validate the fields. Ignore fields which are disabled in
+        # conditionals First get list of fields which are still in the
+        # form after conditionals has be evaluated
+        fields_to_check = self._config.get_fields(values=converted,
                                                   reload_fields=True,
                                                   evaluate=True)
-        for fieldname, field in fields_all:
-            if fieldname not in fields_to_check:
-                continue
-            rules = field.get_rules()
-            for rule in rules:
+        for fieldname, field in fields_to_check.iteritems():
+            field = self.fields[fieldname]
+            for rule in field.get_rules():
                 if rule.mode == "pre":
-                    result = rule.evaluate(submitted)
+                    result = rule.evaluate(unvalidated)
                 else:
-                    result = rule.evaluate(values)
+                    result = rule.evaluate(converted)
                 if not result:
-                    self._add_error(fieldname, rule.msg)
+                    if rule.triggers == "warning":
+                        self._add_warning(fieldname, rule.msg)
+                    else:
+                        self._add_error(fieldname, rule.msg)
 
-        # 6. Custom validation. User defined external validators.
+        # Custom validation. User defined external validators.
         for validator in self.external_validators:
-            if not validator.check(values):
+            if not validator.check(converted):
                 self._add_error(validator._field, validator._error)
 
         # If the form is valid. Save the converted and validated data
@@ -510,9 +570,7 @@ class Form(object):
         # submitted data.
         has_errors = self.has_errors()
         if not has_errors:
-            self.data = values
-        else:
-            self.data = submitted
+            self.data = converted
         self.validated = True
         return not has_errors
 
@@ -558,6 +616,10 @@ class Field(object):
         self._translate = translate
         self.renderer = get_renderer(self, translate)
         self._errors = []
+        self._warnings = []
+        self.value = ""
+        """Value as string of the field. Will be set on rendering the
+        form"""
 
     def __getattr__(self, name):
         """Make attributes from the configuration directly available"""
@@ -599,19 +661,14 @@ class Field(object):
         return "string"
 
     def get_rules(self):
-        """Returns a list of configured rules for the field"""
-        rules = self.rules
-        if self.is_required():
-            parser = Parser()
-            expr = "bool($%s)" % self.name
-            msg = "This field is required. You need to provide a value"
-            mode = "pre"
-            expr = parser.parse(expr)
-            rules.append(Rule(expr, msg, mode))
-        return rules
+        """Returns a list of configured rules for the field."""
+        return self.rules
+
+    def set_value(self, value):
+        self.value = value
 
     def get_value(self, default=None, expand=False):
-        value = self._form.data.get(self._config.name, default)
+        value = self.value
         if expand:
             if not isinstance(value, list):
                 value = [value]
@@ -648,9 +705,15 @@ class Field(object):
     def add_error(self, error):
         self._errors.append(error)
 
+    def add_warning(self, warning):
+        self._warnings.append(warning)
+
     def get_errors(self):
         return self._errors
 
+    def get_warnings(self):
+        return self._warnings
+
     def render(self):
         """Returns the rendererd HTML for the field"""
         return self.renderer.render()
@@ -659,6 +722,10 @@ class Field(object):
         return isinstance(self.sa_property,
                           sa.orm.RelationshipProperty)
 
+    def is_desired(self):
+        """Returns true if field is set as desired in field configuration"""
+        return self.desired
+
     def is_required(self):
         """Returns true if either the required flag of the field
         configuration is set or the field is required in the underlying
