@@ -84,7 +84,7 @@ class FormRenderer(Renderer):
         self.translate = translate
         self.template = template_lookup.get_template("form.mako")
 
-    def render(self, buttons=True):
+    def render(self, buttons=True, values=None):
         """Returns the rendered form as string.
 
         :buttons: Boolean flag to indicate if the form buttons should be
@@ -92,9 +92,11 @@ class FormRenderer(Renderer):
         :returns: rendered form.
 
         """
+        if not values:
+            values = {}
         html = []
         html.append(self._render_form_start())
-        html.append(self._render_form_body())
+        html.append(self._render_form_body(values))
         if not self._form._config.readonly and buttons:
             html.append(self._render_form_buttons())
         html.append(self._render_form_end())
@@ -119,9 +121,10 @@ class FormRenderer(Renderer):
                         % self._form._csrf_token)
         return "".join(html)
 
-    def _render_form_body(self):
+    def _render_form_body(self, values):
         values = {'form': self._form,
-                  '_': self.translate}
+                  '_': self.translate,
+                  'values': values}
         return self.template.render(**values)
 
     def _render_form_buttons(self):
@@ -194,7 +197,9 @@ class FieldRenderer(Renderer):
         # help. Each in its own template.
         html = []
         has_errors = len(self._field.get_errors())
-        html.append('<div class="form-group %s">' % (has_errors and 'has-error'))
+        has_warnings = len(self._field.get_warnings())
+        html.append('<div class="form-group %s %s">' % ((has_errors and 'has-error'),
+                                                       (has_warnings and 'has-warning')))
         html.append(self._render_label())
         values = self._get_template_values()
         html.append(self.template.render(**values))
